/* LIBUSB-WIN32, Generic Windows USB Library
 * Copyright (c) 2002-2005 Stephan Meyer <ste_meyer@web.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "libusb_driver.h"

/* missing in mingw's ddk headers */
#ifndef PLUGPLAY_REGKEY_DEVICE
#define PLUGPLAY_REGKEY_DEVICE  1
#endif

#define LIBUSB_REG_SURPRISE_REMOVAL_OK L"SurpriseRemovalOK"
#define LIBUSB_REG_DEVICE_INTERFACE_GUID L"DeviceInterfaceGuid"


static bool_t reg_get_property(DEVICE_OBJECT *physical_device_object, 
                            int property, char *data, int size);

static bool_t reg_get_property(DEVICE_OBJECT *physical_device_object,
                            int property, char *data, int size)
{
  WCHAR tmp[512];
  ULONG ret;
  ULONG i;

  if(!physical_device_object || !data || !size)
    {
      return FALSE;
    }

  memset(data, 0, size);
  memset(tmp, 0, sizeof(tmp));

  if(NT_SUCCESS(IoGetDeviceProperty(physical_device_object,
                                    property,
                                    sizeof(tmp) - 2,
                                    tmp,
                                    &ret)) && ret)
    {
      /* convert unicode string to normal character string */
      for(i = 0; (i < ret/2) && (i < ((ULONG)size - 1)); i++)
        {
          data[i] = (char)tmp[i];
        }
      
      _strlwr(data);

      return TRUE;
    }

  return FALSE;
}


bool_t reg_get_properties(libusb_device_t *dev)
{
  const ULONG number_of_properties = 2;
  PRTL_QUERY_REGISTRY_TABLE query_reg_table = NULL;
  HANDLE key = NULL;
  NTSTATUS status;

  if(!dev->physical_device_object)
    {
      return FALSE;
    }

  query_reg_table = (PRTL_QUERY_REGISTRY_TABLE)ExAllocatePool(PagedPool,
                                                              sizeof(RTL_QUERY_REGISTRY_TABLE) * (number_of_properties + 1));
  if (!query_reg_table)
    {
      return FALSE;
    }

  /* default settings */
  dev->surprise_removal_ok = FALSE;
  dev->is_filter = TRUE;

  status = IoOpenDeviceRegistryKey(dev->physical_device_object,
                                   PLUGPLAY_REGKEY_DEVICE,
                                   STANDARD_RIGHTS_ALL,
                                   &key);
  if(NT_SUCCESS(status)) 
    {
      const ULONG uninitialized_value = 0xffffffff;
      ULONG surprise_removal_ok = uninitialized_value;
      UNICODE_STRING device_interface_guid_string;

      RtlInitUnicodeString(&device_interface_guid_string, NULL);

      RtlZeroMemory(query_reg_table, sizeof(RTL_QUERY_REGISTRY_TABLE) * (number_of_properties + 1));

      query_reg_table[0].Flags = RTL_QUERY_REGISTRY_DIRECT;
      query_reg_table[0].Name = LIBUSB_REG_SURPRISE_REMOVAL_OK;
      query_reg_table[0].EntryContext = &surprise_removal_ok;
      query_reg_table[0].DefaultType = REG_NONE;

      query_reg_table[1].Flags = RTL_QUERY_REGISTRY_DIRECT;
      query_reg_table[1].Name = LIBUSB_REG_DEVICE_INTERFACE_GUID;
      query_reg_table[1].EntryContext = &device_interface_guid_string;
      query_reg_table[1].DefaultType = REG_NONE;

      status = RtlQueryRegistryValues(RTL_REGISTRY_HANDLE,
                                      (PWSTR)key,
                                      query_reg_table,
                                      NULL,
                                      NULL);

      if (NT_SUCCESS(status))
        {
          if (uninitialized_value != surprise_removal_ok)
            {
              dev->surprise_removal_ok = surprise_removal_ok ? TRUE : FALSE;
              dev->is_filter = FALSE;
            }

          if (device_interface_guid_string.Buffer && (!dev->is_filter))
            {
              status = RtlGUIDFromString((PUNICODE_STRING)&device_interface_guid_string, &dev->device_interface_guid);
              if (NT_SUCCESS(status))
                {
                  dev->device_interface_in_use = TRUE;
                }
              RtlFreeUnicodeString(&device_interface_guid_string);
            }
        }

      ZwClose(key);
    }

  if (query_reg_table)
    {
      ExFreePool(query_reg_table);
    }

  return TRUE;
}

bool_t reg_get_hardware_id(DEVICE_OBJECT *physical_device_object, 
                        char *data, int size)
{
  if(!physical_device_object || !data || !size)
    {
      return FALSE;
    }

  return reg_get_property(physical_device_object, DevicePropertyHardwareID, 
                          data, size);
}

